/****************************************************************************/
/*  Test DMX512                                    */
/****************************************************************************/
/*  LEnsE / Julien VILLEMEJANE       /   Institut d'Optique Graduate School */
/****************************************************************************/
/*  Brochage                                                                */
/*      TO COMPLETE                                                         */
/****************************************************************************/
/*  Test réalisé sur Nucléo-L476RG                                          */
/****************************************************************************/

#include "mbed.h"
#include "platform/mbed_thread.h"

#define SAMPLES     512 
Serial      debug_pc(USBTX, USBRX);

Serial      dmx(A0, A1);
DigitalOut  out_tx(D5);
DigitalOut  start(D4);     //envoie des données
DigitalOut  enableDMX(D6);
AnalogIn    CV_volume(PC_1);
AnalogIn    CV_pitch(PB_0);

AnalogIn    variationR(PC_0);
AnalogIn    variationG(PC_2);
AnalogIn    variationB(PC_3);

// DMX
char dmx_data[SAMPLES] = {0};
char nb = 0;

void initDMX();
void updateDMX();


// MOSI, MISO, SCK, CSN, CE, IRQ
InterruptIn     my_bp(USER_BUTTON);


// Main
int main() {
    debug_pc.baud(9600);
    debug_pc.printf("Essai DMX512\r\n");
    
    initDMX();
    
    while(1) { 
        // Renkforce LPT12 - AD 1
        dmx_data[0] = 0;  
        dmx_data[3] = 100;
        dmx_data[4] = variationR.read() * 255;  
        dmx_data[5] = variationG.read() * 255;   
        dmx_data[6] = variationB.read() * 255;   
        updateDMX();
        wait_us(10000);

    }
}


void initDMX(){
    // Initialisation DMX
    dmx.baud(250000);
    dmx.format (8, SerialBase::None, 2);
    enableDMX = 0;
    // Initialisation canaux DMX
    for(int k = 0; k < SAMPLES; k++){
        dmx_data[k] = 0;
    }    
    updateDMX();  
}

void updateDMX(){
        enableDMX = 1;
        start = 1;      // /start
        out_tx = 0;     // break
        wait_us(88);    
        out_tx = 1;     // mb
        wait_us(8);     
        out_tx = 0;     // break
        start = 0;
        dmx.putc(0);     // Start
        for(int i = 0; i < SAMPLES; i++){
            dmx.putc(dmx_data[i]);     // data
        }
        wait_us(23000); // time between frame  
}
